package fr.plil.sio.persistence.jdbc;

import fr.plil.sio.persistence.api.Right;
import fr.plil.sio.persistence.api.Group;
import fr.plil.sio.persistence.api.User;
import fr.plil.sio.persistence.api.UserService;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import javax.sql.DataSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UserServiceJdbc implements UserService {

    @Autowired
    private UserRepository userRepository;
    
    @Autowired
    private GroupRepository groupRepository;
    
    @Autowired
    private RightRepository rightRepository;

    @Autowired
    private DataSource dataSource;
    
    private static final Logger logger = LoggerFactory.getLogger(UserRepository.class);
    
    @Override
    public User create(String name, String groupName) {
        if(name==null || groupName==null){
            throw new IllegalArgumentException("name cannot be null");
        }
        Group group = new Group();
        group = groupRepository.findByName(groupName);
        User user = userRepository.findByName(name);
        if(group==null){
            throw new IllegalArgumentException("group does not exist");
        }
        if(user!=null){
            throw new IllegalStateException("a user with the same name already exists");
        }
        user = new User();
        user.setName(name);
        
        if(group!=null)
            user.setGroup(group);
     
        userRepository.save(user, group);
        return user;
    }

    @Override
    public boolean delete(String name) {
        if(name == null){
            throw new IllegalArgumentException("name cannot be null");
        }
        User user = findByName(name);
        if(user==null){
            return false;
        }
        userRepository.delete(user.getId());
        return true;
    }

    @Override
    public User findByName(String name) {
        if(name==null){
            throw new IllegalArgumentException("name cannot be null");
        }
        return userRepository.findByName(name);
    }

    @Override
    public boolean isUserHasRight(String userName, Right right) {
        if(userName==null){
            throw new IllegalArgumentException("name cannot be null");
        }
        if(right==null){
            throw new IllegalArgumentException("right cannot be null");
        }
        if(rightRepository.findByName(right.getName()).size()==0){
            throw new IllegalArgumentException("right not in database");
        }
        return userRepository.isUserHasRight(userRepository.findByName(userName).getId(), right.getId());
    }
}
